/*
Copyright (c) 2017, Blueskyenergyinc

All rights reserved.
 */

#ifndef DATA_H_
#define DATA_H_

#include "nrf_queue.h"

// IPN DATA
typedef enum {
	BATTERY_VOLTAGE = 1,
	NET_BATTERY_CURRENT = 2,
	TOTAL_INPUT_CURRENT = 3,
	TOTAL_OUTPUT_CURRENT = 4,
	BATTERY_REMAINING = 5,
	BATTERY_FROM_FULL = 6,
	AUX_OUTPUT_MODE = 7,
	AUX_OUTPUT_VOLTAGE = 8,
	AUX_OUTPUT_STATE = 9,
	CHARGE_STATE = 10,
	START_EQUALIZE = 11,
	STOP_EQUALIZE = 12,
	EQUALIZE_TIME_LEFT = 13,
	DAYS_SINCE_CHARGE = 14,
	DAYS_SINCE_EQUALIZE = 15,
	VBAT_MAX = 16,
	CLEAR_VBAT_MAX = 17,
	VBAT_MIN = 18,
	CLEAR_VBAT_MIN = 19,
	TOTAL_CHARGE_AHR = 20,
	CLEAR_TOTAL_CHG_AHR = 21,
	LIFETIME_BAT = 22,
	CLEAR_LIFETIME_BAT = 23,
	BAT_TEMP = 24,
	UNIT_IN_V = 25,
	UNIT_IN_A = 26,
	UNIT_OUT_A = 27,
	UNIT_TEMP = 28,
	UNIT_OUT_AHR = 29,
	BAT_AH_SP = 30,
	SET_BAT_AH_SP = 31,
	CHARGE_EFF_SP = 32,
	SET_CHARGE_EFF_SP = 33,
	CHARGE_EFF_MODE = 34,
	SELF_DISCHG_SP = 36,
	SET_SELF_DISCHG_SP = 37,
	MAX_BAT_V = 41,
	AUX_MODE = 43,
	AUX_OFF_V = 45,
	AUX_ON_V = 47,
	AUX_OFF_AHR = 49,
	AUX_ON_AHR = 51,
	RESET_CC_DEFAULTS = 53,
	ACCEPT_V = 54,
	FLOAT_V = 56,
	EQUALIZE_V = 58,
	EQUALIZE_TIME = 60,
	EQUALIZE_DAYS = 62,
	TEMP_COMP = 64,
	FLOAT_AMPS = 66,
	ACCEPT_TIME = 68,
	POST_DUSK_HRS = 70,
	SET_POST_DUSK_HRS = 71,
	PRE_DAWN_HRS = 72,
	SET_PRE_DAWN_HRS = 73
} IPN_DATA_TYPE;

typedef struct ipn_req_data_s {
        uint8_t index;
        uint8_t unit;
        uint16_t value;
} IPN_REQ_DATA_S;

typedef struct ipn_send_data_s {
        uint8_t index;
        uint8_t unit;
        uint16_t value;
        const char* text;
} IPN_SEND_DATA_S;

typedef void (*u_cb_t)(IPN_DATA_TYPE, uint8_t*, uint16_t*);  // Update Callback
typedef void (*r_cb_t)(IPN_REQ_DATA_S*);  // Request Callback

typedef struct ipn_data_s {
        const uint8_t index;
        uint8_t unit;
        uint16_t value;
        const char* text;
        u_cb_t u_cb;
        r_cb_t r_cb;
} IPN_DATA_S;

#define MAX_NUM_UNITS 8  // There are only 8 IPN Ctrls including master
typedef struct ipn_unit_data_s {
        uint16_t inv;
        uint16_t ina;
        uint16_t outa;
        uint16_t outahr;
        uint16_t temp;
} IPN_UNIT_DATA_S;

#define IPN_DATA_SIZE           74
extern IPN_DATA_S ipn_data[IPN_DATA_SIZE];  // Holds all the IPN data
extern IPN_UNIT_DATA_S ipn_unit_data[MAX_NUM_UNITS];  // Hold all the Unit Data
#define IPN_LOG_DATA_SIZE		33
#define UNIT_OFFSET				9
#define	UNIT_SIZE_OFFSET		3
#define MAXINV					0
#define MAXOUTA					1
#define CHGAH					2
#define RECDTIME				8
#define ACCPTIME				2
#define FLOATTIME				3
extern uint16_t ipn_log_data[IPN_LOG_DATA_SIZE];  // Holds all the Log data
void reset_ipn_log_data();
void one_min_check_log_data();


// IPN Messages
#define IPN_MAX_MSG_SIZE		36
typedef struct ipn_msg_s {
	char msg[IPN_MAX_MSG_SIZE];
	uint8_t length;
} IPN_MSG_S;

// IPN Status Keep Alive Timeouts
#define IPN_TIMEOUT	4	// Four seconds
uint32_t ipn_messages_timeout;
uint32_t pro_remote_timeout;
uint32_t ucm_timeout;

// Queue Interface DEC go here, DEF go in main
NRF_QUEUE_INTERFACE_DEC(IPN_MSG_S, ipn_msg_queue);
NRF_QUEUE_INTERFACE_DEC(IPN_REQ_DATA_S, ble_req_queue);
NRF_QUEUE_INTERFACE_DEC(IPN_SEND_DATA_S, ble_send_queue);

#endif /* DATA_H_ */
